#
# $Id: z39util.tcl,v 1.29 1996/01/31 15:56:37 adam Exp $
#
proc saveState {} {
    uplevel #0 {
    set f [open "tcl.state.${sessionId}" w]
    foreach var [info globals] {
        if {$var == "f"} continue
        if {$var == "sessionId"} continue
        if {$var == "errorInfo"} continue
	if {[catch {set names [array names $var]}]} {
            eval "set v \$${var}"
            puts $f "set ${var} \{$v\}"
        } else {
	    foreach n $names {
                eval "set v \$${var}(\$n)"
		puts $f "set ${var}($n) \{$v\}"
	    }
            catch {
                eval "set v \$${var}"
                puts $f "set ${var} \{$v\}"
            }
        }
    }
    close $f
    }
}

proc search-response {zz} {
    global sessionWait

    set status [$zz responseStatus]
    if {[lindex $status 0] == "NSD"} {
        $zz nextResultSetPosition 0
        set code [lindex $status 1]
        set msg [lindex $status 2]
        set addinfo [lindex $status 3]
        displayError "Diagnostic message" \
                "$msg: $addinfo<br>\n(error code $code)"
        set sessionWait -2
    } else {
        set sessionWait 1
    }
}

proc scan-response {zz} {
    global sessionWait

    set status [$zz scanStatus]
    if {$status == 6} {
        displayError "Scan fail" ""
        set sessionWait -2
    } else {
        set sessionWait 1
    }
}

proc ok-response {} {
    global sessionWait
    set sessionWait 1
}

proc fail-response {} {
    global sessionWait
    set sessionWait -1
}

proc display-brief {zset no tno} {
    global env
    global setNo
    global sessionId


    html {<li>}
    set type [$zset type $no]
    if {$type == "SD"} {
        set err [lindex [$zset diag $no] 1]
        set add [lindex [$zset diag $no] 2]
        if {$add != {}} {
            set add " :${add}"
        }
        html "${no} Error ${err}${add} <br>\n"
        return
    }
    if {$type != "DB"} {
        return
    }
    set rtype [$zset recordType $no]
    if {$rtype == "SUTRS"} {
        html [join [$zset getSutrs $no]]
        html "<br>\n"
        return
    } 
    if {![catch {
        set author [$zset getMarc $no field 100 * a]
        set corp [$zset getMarc $no field 110 * a]
        set meet [$zset getMarc $no field 111 * a]
        set title [$zset getMarc $no field 245 * a]
	if {[llength $author] == 0} {
            set cover [$zset getMarc $no field 245 * {[bc]}]
	} else {
            set cover [$zset getMarc $no field 245 * b]
	}
	set location [$zset getMarc $no field 260 * a] 
	set publisher [$zset getMarc $no field 260 * b]
        set year [$zset getMarc $no field 260 * c]
    } ] } {
        html { <a href="http:} $env(SCRIPT_NAME) /
        html $sessionId {/showfull.egw/} $setNo + $tno + $no + full {">}
        set p 0
        foreach a $author {
            if {$p} {
                html ", "
            }
            html $a
            set p 1
        }
        foreach a $corp {
            if {$p} {
                html ", "
            }
            html $a
            set p 1
        }
        foreach a $meet {
            if {$p} {
                html ", "
            }
            html $a
            set p 1
        }
        if {$p} {
            html ": "
        }
	set nope 1
        foreach v $title {
            html $v
            set nope 0
        }
        if {$nope} {
            set v [join $cover ""]
            if {[string length $v] > 40} {
                html [string range $v 0 38] "..."
            } else {
                html $v
            }
	}
        html {</a> }
    }
    html "<br>\n"
}

proc display-raw {zset no tno} {
    set type [$zset type $no]
    if {$type == "SD"} {
        set err [lindex [$zset diag $no] 1]
        set add [lindex [$zset diag $no] 2]
        if {$add != {}} {
            set add " :${add}"
        }
        html "<h3>${no}</h3>\n"
        html "Error ${err}${add} <br>\n"
        return
    }
    if {$type != "DB"} {
        return
    }
    set rtype [$zset recordType $no]
    if {$rtype == "SUTRS"} {
        html [join [$zset getSutrs $no]] "<br>\n"
        return
    } 
    if {[catch {set r [$zset getMarc $no line * * *]}]} {
        html "Unknown record type: $rtype <br>\n"
        return
    }
    foreach line $r {
        set tag [lindex $line 0]
        set indicator [lindex $line 1]
        set fields [lindex $line 2]
        set l [string length $indicator]
        html "<tt>$tag "
        if {$l > 0} {
            for {set i 0} {$i < $l} {incr i} {
                if {[string index $indicator $i] == " "} {
                    html "-"
                } else {
                    html [string index $tag $i]
                }
            }
        }
        html "</tt>"
        foreach field $fields {
            set id [lindex $field 0]
            set data [lindex $field 1]
            if {$id != ""} {
                html " <b>\$$id</b> "
            }
            html $data
        }
        html "<br>\n"
    }
}

proc put-marc-contents {cc} {
    set ref ""
    if {[string first :// $cc] > 0} {
        foreach urltype {gopher http ftp mailto} {
            if {[string first ${urltype}:// $cc] == 0} {
                set ref $urltype
                break
            }
        }
    } 
    if {$ref != ""} {
        html {<a href="}
    }
    html $cc
    if {$ref != ""} {
        html {">} $cc {</a>}
    }
}

proc dl-marc-field {zset no tag id la lb sep} {
    set n 0
    set c [$zset getMarc $no field $tag * $id]
    set len [llength $c]
    if {$len == 0} {
        return 0
    }
    if {$len > 1 && "x$lb" != "x"} {
        html "<dt><b>$lb</b>\n<dd>"
    } else {
        html "<dt><b>$la</b>\n<dd>"
    }
    foreach cc $c {
        if {$n > 0} {
            html $sep
        }
        put-marc-contents $cc
        incr n
    }
    return $n
}

proc dd-marc-field {zset no tag id start stop} {
    set n 0
    set c [$zset getMarc $no field $tag * $id]
    set len [llength $c]
    if {$len == 0} {
        return 0
    }
    foreach cc $c {
        html $start
        put-marc-contents $cc
        html $stop
        incr n
    }
    return $n
}

proc dl-marc-field-rec {zset no tag lead start stop startid sep} {
    set n 0
    set lines [$zset getMarc $no line $tag * *]
    foreach line $lines {
        foreach field [lindex $line 2] {
            if {$n == 0} {
                html "<dt><b>$lead</b>"
                html "\n<dd>"
            }
            set id [lindex $field 0]
            if {$id == $startid} {
                if {$n > 0} {
                    html $stop
                }
                html $start
                incr n
                html [lindex $field 1]
            } else {
                html $sep
                html [lindex $field 1]
            }
        }
    }
    if {$n > 0} {
        html $stop
    }
}

proc display-full {zset no tno} {
    set type [$zset type $no]
    if {$type == "SD"} {
        set err [lindex [$zset diag $no] 1]
        set add [lindex [$zset diag $no] 2]
        if {$add != {}} {
            set add " :${add}"
        }
        html "Error ${err}${add} <br>\n"
        return
    }
    if {$type != "DB"} {
        return
    }
    set rtype [$zset recordType $no]
    if {$rtype == "SUTRS"} {
        html [join [$zset getSutrs $no]] "<br>\n"
        return
    } 
    if {[catch {set r [$zset getMarc $no line * * *]}]} {
        html "Unknown record type: $rtype <br>\n"
        return
    }
    html "<dl>\n"
    set n [dl-marc-field $zset $no 700 a "Author" "Authors" "<br>\n"]
    if {$n == 0} {
        set n [dl-marc-field $zset $no 100 a "Author" "Authors" "<br>\n"]
    }
    set n [dl-marc-field $zset $no 710 a "Corporate Name" {} ", "]
    if {$n == 0} {
        set n [dl-marc-field $zset $no 110 a "Corporate Name" {} ", "]
    }
    set n [dl-marc-field $zset $no 711 a "Meeting Name" {} ", "]
    if {$n > 0} {
	dd-marc-field $zset $no 711 {[bndc]} " " ""
    } else {
        set n [dl-marc-field $zset $no 111 a "Meeting Name" {} ", "]
	if {$n > 0} {
	    dd-marc-field $zset $no 111 {[bndc]} " " " "
	}
    } 
    set n [dl-marc-field $zset $no 245 {a} "Title" {} " "]
    if {$n > 0} {
        dd-marc-field $zset $no 245 b "<em>" "</em>"
        dd-marc-field $zset $no 245 c " " ""
    } else {
        dl-marc-field $zset $no 245 {[ab]} "Title" {} " "
    }
    dl-marc-field $zset $no 520 a "Abstract" {} ", "
    dl-marc-field $zset $no 072 * "Subject code" "Subject codes" ", "
    dl-marc-field $zset $no 650 * "Subject" {} ", "
    dl-marc-field $zset $no 260 * "Publisher" {} " "
    dl-marc-field $zset $no 300 * "Physical Description" {} " "

    dl-marc-field-rec $zset $no 500 "Notes" "" "<br>\n" "a" ", "

    dl-marc-field-rec $zset $no 510 "References" "" "<br>\n" "a" ", "

    dl-marc-field-rec $zset $no 511 "Participant note" "" "<br>\n" "a" ", "

    dl-marc-field $zset $no 513 a "Report type" {} ", "
    dl-marc-field $zset $no 513 b "Period covered" {} ", "
    dl-marc-field-rec $zset $no 515 "Numbering notes" "" "<br>\n" "a" ", "
    dl-marc-field-rec $zset $no 516 "Data notes" "" "<br>\n" "a" ", "
    dl-marc-field-rec $zset $no 518 "Date/time notes" "" "<br>\n" "a" ", "

    dl-marc-field $zset $no 350 a "Price" {} ", "
    dl-marc-field $zset $no 362 a "Dates of publication" {} ", "
    dl-marc-field $zset $no 850 a "Holdings" {} ", "

    dl-marc-field-rec $zset $no 270 "Contact name" "" "<br>\n" p ", "
    if {0} {
        set n [dl-marc-field $zset $no 270 p "Contact name" {} ", "]
        if {$n > 0} {
            html "\n<dl>\n"
            
            if {0} {
                dl-marc-field $zset $no 270 a "Street" {} ", "
                dl-marc-field $zset $no 270 b "City" {} ", "
                dl-marc-field $zset $no 270 c "State" {} ", "
                dl-marc-field $zset $no 270 e "Zip code" {} ", "
                dl-marc-field $zset $no 270 d "Country" {} ", "
                dl-marc-field $zset $no 270 m "Network address" {} ", "
                dl-marc-field $zset $no 301 a "Service hours" {} ", "
                dl-marc-field $zset $no 270 k "Phone" {} ", "
                dl-marc-field $zset $no 270 l "Fax" {} ", "
            } else {
                dl-marc-field $zset $no 270 {[abcedmakl]} "Address" {} "<br>\n"
            }
            
            html "\n</dl>\n"
        }
    }
    dl-marc-field $zset $no 010 a "LC control number" {} ", "
    dl-marc-field $zset $no 010 b "NUCMC control number" {} ", "
    dl-marc-field $zset $no 020 a "ISBN" {} ", "
    dl-marc-field $zset $no 022 a "ISSN" {} ", "
    set url [$zset getMarc $no field 856 * u]
    set sp [$zset getMarc $no field 856 * 3]
    if {"x$url" != "x"} {
        html "<dt><b>URL</b>\n"
        if {"x$sp" == "x"} {
            set sp $url
        }
        html {<dd><a href="} $url {">} [join $sp] "</a>\n"
    }
    dl-marc-field $zset $no 037 {[abc]} "Acquisition" {} "<br>\n"
    dl-marc-field $zset $no 037 {[f6]} "Form of issue" {} "<br>\n"
    dl-marc-field $zset $no 537 * "Source of data" {} "<br>\n"
    dl-marc-field $zset $no 538 * "System details" {} "<br>\n"
    dl-marc-field $zset $no 787 {[rstw6]} "Related information" {} "<br>\n"
    dl-marc-field $zset $no 001 * "Local control number" {} ", "
    html "</dl>\n"
}


proc display-rec {from to dfunc tno} {
    global setNo

    if {$tno > 0} {
        while {$from <= $to} { 
            eval "$dfunc z39${tno}.${setNo} $from $tno"
            incr from
        }
    } else {
        while {$from <= $to} { 
            eval "$dfunc z39.${setNo} $from 0"
            incr from
        }
    }
}

proc build-scan {t i} {
    global targets

    set term [egw_form entry$i]
    if {$term != ""} {
        set field [join [egw_form menu$i]]
        set attr {Title}
        foreach x [lindex $targets($t) 2] {
            if {[lindex $x 0] == $field} {
                set attr [lindex $x 1]
            }
        }
        return [list $term $attr]
    }
    return ""
}

proc build-query {t ilines} {
    global targets

    set op {}
    set q {}
    for {set i 1} {$i <= $ilines} {incr i} {
        set term [join [egw_form entry$i]]
        if {[string length $term] > 0} {
            set field [join [egw_form menu$i]]
            foreach x [lindex $targets($t) 2] {
                if {[lindex $x 0] == $field} {
                    set attr [lindex $x 1]
                }
            }
            switch $op {
            And
                { set q "@and $q ${attr} \"${term}\"" }
            Or
                { set q "@or $q ${attr} \"${term}\"" }
            {And not}
                { set q "@not $q ${attr} \"${term}\"" }
            {}
                { set q "${attr} \"${term}\"" }
            }
            set op [egw_form logic$i]
        }
    }
    return $q
}

proc z39scan {setNo scanNo tno scanLines scanPos cache} {
    global hist
    global sessionWait
    global targets

    if {$tno > 0} {
        set zz z39$tno
        set host $hist($setNo,$tno,host)
        set idAuth $hist($setNo,$tno,idAuthentication)
        set database $hist($setNo,$tno,database)
        set scanAttr $hist($setNo,$tno,scanAttr)
        set scanTerm $hist($setNo,$tno,$scanNo,scanTerm)
    } else {
        set zz z39
        set host $hist($setNo,host)
        set idAuth $hist($setNo,idAuthentication)
        set database $hist($setNo,database)
        set scanAttr $hist($setNo,scanAttr)
        set scanTerm $hist($setNo,$scanNo,scanTerm)
    }
    if {[catch [list $zz failback fail-response]]} {
        ir $zz
    }
    if {[catch [list set oldHost [$zz connect]]]} {
        set oldHost ""
    }
    set zs $zz.s$scanNo.$setNo
    $zz callback ok-response
    $zz failback fail-response
    set thisHost [splitHostSpec $host]
    if {$oldHost != $thisHost} {
        catch [list $zz disconnect]

        set sessionWait 0
        if {[catch [list $zz connect $thisHost]]} {
            displayError "Cannot connect to target" $thisHost
            return 0
        } elseif {$sessionWait == 0} {
            if {[catch {egw_wait sessionWait 300}]} {
		$zz disconnect
                displayError "Cannot connect to target" $thisHost
                return 0
            }
            if {$sessionWait != 1} {
                displayError "Cannot connect to target" $thisHost
                return 0
            }
        }
        $zz idAuthentication $idAuth
        set sessionWait 0
        if {[catch {$zz init}]} {
            displayError "Cannot initialize target" $thisHost
	    $zz disconnect
            return 0
        }
        if {[catch {egw_wait sessionWait 60}]} {
            displayError "Cannot initialize target" $thisHost
	    $zz disconnect
            return 0
        }
        if {$sessionWait != "1"} {
            displayError "Cannot initialize target" $thisHost
	    $zz disconnect
            return 0
        }
        if {![$zz initResult]} {
            set u [$zz userInformationField]
            $zz disconnect
            displayError "Cannot initialize target $thisHost" $u
            return 0
        }
    } else {
        if {$cache && ![catch [list $zs numberOfTermsRequested 5]]} {
            return 1
        }
    }
    eval $zz databaseNames $database

    ir-scan $zs $zz

    $zs numberOfTermsRequested $scanLines
    $zs preferredPositionInResponse $scanPos

    $zz callback [list scan-response $zs]

    egw_log debug "scan: ${scanAttr} ${scanTerm}"
    set sessionWait 0
    $zs scan "${scanAttr} ${scanTerm}"

    if {[catch {egw_wait sessionWait 60}]} {
        egw_log debug "timeout/cancel in scan"
        displayError "Timeout in scan" {}
        html "</body></html>\n"
        $zz disconnect
        return 0
    }
    if {$sessionWait == -1} {
        displayError "Scan fail" "Connection closed"
        html "</body></html>\n"
        $zz disconnect
    }
    if {$sessionWait != 1} {
	return 0
    }
    return 1
}

proc display-scan {setNo scanNo tno} {
    global hist
    global targets
    global env
    global sessionId

    if {$tno > 0} {
        set zz z39$tno
    } else {
        set zz z39
    }
    set zs $zz.s$scanNo.$setNo
    set m [$zs numberOfEntriesReturned]
        
    if {$m > 0} {
        set t [lindex [$zs scanLine 0] 1]
        if {$tno > 0} {
            set hist($setNo,$tno,[expr $scanNo - 1],scanTerm) $t
        } else {
            set hist($setNo,[expr $scanNo - 1],scanTerm) $t
        }
        set t [lindex [$zs scanLine [expr $m - 1]] 1]
        if {$tno > 0} {
            set hist($setNo,$tno,[expr $scanNo + 1],scanTerm) $t
        } else {
            set hist($setNo,[expr $scanNo + 1],scanTerm) $t
        }
    }
    html {<table width=500 border=0><tr>}
    html {<td align=left><b>Scan term</b>}
    html {<td align=right><b>Hits</b>}
    html {<tr>} \n

    for {set i 0} {$i < $m} {incr i} {
        html {<td align=left>}
        if {0} {
            regsub -all {\ } [lindex [$zs scanLine $i] 1] + tterm
            html {<a href="http:} $env(SCRIPT_NAME)
            html / $sessionId {/query.egw/} $hist($setNo,host) + $setNo +
            html $hist($setNo,scan) +  $tterm {">}
        } else {
            regsub -all {\ } [lindex [$zs scanLine $i] 1] + tterm
            html {<a href="http:} $env(SCRIPT_NAME)
            html / $sessionId {/search.egw/} $setNo +
            html hyper + $tterm {">}
        }
        html [lindex [$zs scanLine $i] 1]
        html {</a>} 
        html {<td align=right>}
        html [lindex [$zs scanLine $i] 2]
        html {<tr>} \n
    }
    html {</table} \n
}

proc z39search {setNo piggy tno elements} {
    global hist
    global sessionWait
    global targets

    if {$tno > 0} {
        set zz z39$tno
        set host $hist($setNo,$tno,host)
        set idAuth $hist($setNo,$tno,idAuthentication)
        set database $hist($setNo,$tno,database)
        set query $hist($setNo,$tno,query)
    } else {
        set zz z39
        set host $hist($setNo,host)
        set idAuth $hist($setNo,idAuthentication)
        set database $hist($setNo,database)
        set query $hist($setNo,query)
    }
    if {[catch [list $zz failback fail-response]]} {
        ir $zz
    }
    if {[catch [list set oldHost [$zz connect]]]} {
        set oldHost ""
    }
    $zz callback ok-response
    $zz failback fail-response
    set thisHost [splitHostSpec $host]
    if {$oldHost != $thisHost} {
        catch [list $zz disconnect]

        set sessionWait 0
        if {[catch [list $zz connect $thisHost]]} {
            displayError "Cannot connect to target" $thisHost
            return 0
        } elseif {$sessionWait == 0} {
            if {[catch {egw_wait sessionWait 300}]} {
		$zz disconnect
                displayError "Cannot connect to target" $thisHost
                return 0
            }
            if {$sessionWait != 1} {
                displayError "Cannot connect to target" $thisHost
                return 0
            }
        }
        $zz idAuthentication $idAuth
        set sessionWait 0
        if {[catch {$zz init}]} {
            displayError "Cannot initialize target" $thisHost
	    $zz disconnect
            return 0
        }
        if {[catch {egw_wait sessionWait 60}]} {
            displayError "Cannot initialize target" $thisHost
	    $zz disconnect
            return 0
        }
        if {$sessionWait != "1"} {
            displayError "Cannot initialize target" $thisHost
	    $zz disconnect
            return 0
        }
        if {![$zz initResult]} {
            set u [$zz userInformationField]
            $zz disconnect
            displayError "Cannot initialize target $thisHost" $u
            return 0
        }
    } else {
        if {[info exists hist($setNo,hits)] && \
                ![catch [list $zz.$setNo smallSetUpperBound 0]]} {
            return 1
        }
        
    }
    ir-set $zz.$setNo $zz
    
    if {![lindex $targets($host) 5]} {
        set elements {}
    }
    $zz.$setNo smallSetElementSetNames $elements
    $zz.$setNo mediumSetElementSetNames $elements
    $zz.$setNo recordElements $elements

    egw_log debug "database=$database"
    eval $zz.$setNo databaseNames $database

    $zz.$setNo preferredRecordSyntax USMARC

    $zz callback [list search-response $zz.$setNo]
    if {$piggy} {
        $zz.$setNo largeSetLowerBound 999999
        $zz.$setNo smallSetUpperBound 0
        $zz.$setNo mediumSetPresentNumber $hist($setNo,maxPresent)
    } else {
        $zz.$setNo largeSetLowerBound 2
        $zz.$setNo smallSetUpperBound 0
        $zz.$setNo mediumSetPresentNumber 0
    }
    set sessionWait 0
    egw_log debug "search: $query"
    $zz.$setNo search $query

    if {[catch {egw_wait sessionWait 600}]} {
        egw_log debug "timeout/cancel in search"
        displayError "Timeout in search" {}
        html "</body></html>\n"
        $zz disconnect
        return 0
    }
        
    if {$sessionWait == -1} {
        displayError "Search fail" "Connection closed"
        html "</body></html>\n"
        $zz disconnect
    }
    if {$sessionWait != 1} {
	return 0
    }
    set hist($setNo,hits) [$zz.$setNo resultCount]
    return 1
}

proc init-m-response {i} {
    global zstatus
    global zleft

    egw_log debug "init-m-response"

    set zstatus($i) 1
    incr zleft -1
}

proc connect-m-response {i} {
    global zstatus
    global zleft

    egw_log debug "connect-m-response"
    z39$i callback [list init-m-response $i]
    if {[catch {z39$i init}]} {
        set zstatus($i) -1
        incr zleft -1
    }
}

proc fail-m-response {i} {
    global zstatus
    global zleft
    
    egw_log debug "fail-m-response"
    set zstatus($i) -1
    incr zleft -1
}

proc search-m-response {setNo i start number} {
    global zleft
    global zstatus
    global hist

    egw_log debug "search-m-response"
    set status [z39$i.$setNo responseStatus]
    egw_log debug "search-m-response1"
    if {[lindex $status 0] != "DBOSD"} {
        egw_log debug "search-m-response2"
        incr zleft -1
        set zstatus($i) 2
        return
    }
    set nor [z39$i.$setNo numberOfRecordsReturned]
    egw_log debug "search-m-response3"
    set hist($setNo,$i,offset) [expr $start + $nor -1]
    if {[expr $nor + $start] >= [z39$i.$setNo resultCount]} {
        egw_log debug "search-m-response4"
        incr zleft -1
        set zstatus($i) 2
        return
    }
    egw_log debug "search-m-response5"
    if {$nor >= $number} {
        egw_log debug "search-m-response6"
        incr zleft -1
        set zstatus($i) 2
        return
    }
    egw_log debug "search-m-response7"
    set start [expr $start + $nor]
    set number [expr $number - $nor]
    if {[expr $start + $number - 1] > [z39$i.$setNo resultCount]} {
        set number [expr [z39$i.$setNo resultCount] - $start + 1]
    }
    z39$i callback [list search-m-response $setNo $i $start $number]
    egw_log debug "mpresent start=$number number=$number"
    z39$i.$setNo present $start $number
}

proc z39msearch {setNo elements start number cache} {
    global zleft
    global zstatus
    global hist
    global targets

    set not $hist($setNo,0,host)

    for {set i 1} {$i <= $not} {incr i} {
        set host $hist($setNo,$i,host)
        if {[catch [list z39$i failback fail-m-response $i]]} {
            ir z39$i
        }
        set oldHost [z39$i connect]
        set thisHost [splitHostSpec $host]
        if {$oldHost != $thisHost} {
            catch {z39$i disconnect}
        }
        z39$i callback [list connect-m-response $i]
        z39$i failback [list fail-m-response $i]
    }
    set zleft 0
    for {set i 1} {$i <= $not} {incr i} {
        set oldHost [z39$i connect]
        set host $hist($setNo,$i,host)
        set thisHost [splitHostSpec $host]
        if {$oldHost == $thisHost} {
            continue
        }
        egw_log debug "old=$oldHost this=$thisHost"
        z39$i idAuthentication $hist($setNo,$i,idAuthentication)
        html "Connecting to target " $thisHost " <br>\n"
        set zstatus($i) -1
        if {![catch {z39$i connect $thisHost}]} {
            incr zleft
        } 
    }
    while {$zleft > 0} {
        egw_log debug "Waiting for init response"
        if {[catch {egw_wait zleft 20}]} {
            break
        }
    }
    set zleft 0
    for {set i 1} {$i <= $not} {incr i} {
        html "host " [splitHostSpec $hist($setNo,$i,host)] ": "
        egw_log debug "i=$i zstatus=$zstatus($i)"
        if {$zstatus($i) < 1} {
            html "fail<br>\n"
            continue
        }
        if {[catch [list z39$i.$setNo preferredRecordSyntax USMARC]]} {
            html "ok<br>\n"
            ir-set z39$i.$setNo z39$i
            set hist($setNo,$i,offset) 0
            eval z39$i.$setNo databaseNames $hist($setNo,$i,database)

            if {![lindex $targets($hist($setNo,$i,host)) 5]} {
                set thisElements {}
            } else {
                set thisElements $elements
            }
            z39$i.$setNo smallSetElementSetNames $thisElements
            z39$i.$setNo mediumSetElementSetNames $thisElements
            z39$i.$setNo elementSetNames $thisElements
            z39$i.$setNo recordElements $thisElements

            z39$i.$setNo preferredRecordSyntax USMARC
            z39$i callback [list search-m-response $setNo $i $start $number]

            if {$start == 1} {
                z39$i.$setNo largeSetLowerBound 999999
                z39$i.$setNo smallSetUpperBound 0
                z39$i.$setNo mediumSetPresentNumber $number
            } else {
                z39$i.$setNo largeSetLowerBound 2
                z39$i.$setNo smallSetUpperBound 0
                z39$i.$setNo mediumSetPresentNumber 0
            }
            set zstatus($i) 1
            incr zleft
            egw_log debug "setNo=$setNo msearch " $hist($setNo,$i,query)
            z39$i.$setNo search $hist($setNo,$i,query)
        } elseif {[z39$i.$setNo resultCount] >= $start} {
            if {[expr $start + $number - 1] > [z39$i.$setNo resultCount]} {
                set tnumber [expr [z39$i.$setNo resultCount] - $start + 1]
            } else {
                set tnumber $number
            }
            if {![lindex $targets($hist($setNo,$i,host)) 5]} {
                set thisElements {}
            } else {
                set thisElements $elements
            }
            z39$i.$setNo smallSetElementSetNames $thisElements
            z39$i.$setNo mediumSetElementSetNames $thisElements
            z39$i.$setNo elementSetNames $thisElements
            z39$i.$setNo recordElements $thisElements

            for {set n 0} {$n < $tnumber} {incr n} {
                if {[z39$i.$setNo type [expr $start + $n]] == ""} {
                    if {$n > 0} {
                        egw_log debug "failed on $n"
                    }
                    break
                }
            }
            if {$n == $tnumber} {
                html "cached<br>\n"
                continue
            }
            
            html "present<br>\n"
            z39$i.$setNo preferredRecordSyntax USMARC
            z39$i callback [list search-m-response $setNo $i $start $tnumber]
            incr zleft
            egw_log debug "mpresent start=$start number=$tnumber"
            z39$i.$setNo present $start $tnumber
        } else {
            html "ok<br>\n"
        }
    }
    while {$zleft > 0} {
        egw_log debug "Waiting for search/present response"
        if {[catch {egw_wait zleft 60}]} {
            break
        }
    }
    for {set i 1} {$i <= $not} {incr i} {
        if {$zstatus($i) != 2} continue
        set status [z39$i.$setNo responseStatus]
        if {0 && [lindex $status 0] != "NSD"} {
            set hist($setNo,$i,offset) [z39$i.$setNo numberOfRecordsReturned]
        }
    }
}

proc z39present {setNo tno setOffset setMax dfunc elements} {
    global hist
    global sessionWait
    global targets

    if {$tno > 0} {
        set zz z39$tno
        set host $hist($setNo,$tno,host)
    } else {
        set zz z39
        set host $hist($setNo,host)
    }

    if {![lindex $targets($host) 5]} {
        set elements {}
    }

    $zz.$setNo elementSetNames $elements
    $zz.$setNo recordElements $elements
    set toGet [expr 1 + $setMax - $setOffset]

    $zz callback [list search-response $zz.$setNo]

    while {$setMax > 0 && $toGet > 0} {
        for {set got 0} {$got < $toGet} {incr got} {
            if {[$zz.$setNo type [expr $setOffset + $got]] == ""} {
                break
            }
        }
        if {$got < $toGet} {
            set sessionWait 0
            $zz.$setNo present $setOffset $toGet
            if {[catch {egw_wait sessionWait 300}]} {
                egw_log debug "timeout/cancel in present"
		$zz disconnect
                break
	    }
            if {$sessionWait == "0"} {
                $zz disconnect
            }
            if {$sessionWait != "1"} {
                break
            }
            set got [$zz.$setNo numberOfRecordsReturned]
            if {$got <= 0} {
                break
            }
        }
        display-rec $setOffset [expr $got + $setOffset - 1] $dfunc $tno
        set setOffset [expr $got + $setOffset]
        set toGet [expr 1 + $setMax - $setOffset]
        egw_flush
    }
}

proc z39history {} {
    global nextSetNo
    global hist
    global env
    global sessionId
    global targets
    global html3

    if {![info exists nextSetNo]} {
        return
    }
    html "<h2>History</h2><br>\n"
    if {$html3} {
        html {<table width=500 border=1><tr>}
        html {<td align=center><b>Target</b>}
        html {<td align=center><b>Database</b>}
        html {<td align=center><b>Hits</b>}
        html {<td align=center><b>Query</b>}
        html {<tr>} "\n"
    } else {
        html {<dl>} "\n"
    }
    for {set setNo 1} {$setNo < $nextSetNo} {incr setNo} {
        if {$hist($setNo,scan) > 0} continue
        set host $hist($setNo,host)
        if {$html3} {
            html {<td align=left>}
        } else {
            html {<dt> }
        }
        html [lindex $targets($host) 0]
        if {$html3} {
            html {<td align=left>} [join $hist($setNo,database)]
        } else {
            if {[llength [lindex $targets($host) 1]] > 1} {
                html ": "
                foreach b $hist($setNo,database) {
                    html " $b"
                }
            }
            html {. }
        }
        if {$html3} {
            html {<td align=right>}
        }
        if {[info exists hist($setNo,hits)]} {
            html { <a href="http:} $env(SCRIPT_NAME)
            html / $sessionId {/search.egw/} $setNo + 1
            html + $hist($setNo,maxPresent)
            if {1} {
                html {">} $hist($setNo,hits) {</a>}
            } else {
                html {">Result</a>: } $hist($setNo,hits) { hits.}
            }
        } else {
            if {$html3} {
                html {Failed}
            } else {
                html {Search failed.}
            }
        }
        if {$html3} {
            html {<td align=left>}
        } else {
            html "<dd>\n"
        }
        html { <a href="http:} $env(SCRIPT_NAME)
        html / $sessionId {/query.egw/} $host + $setNo
        if {$html3} {
            html {">}
        } else {
            html {">Query</a>: }
        }
        set op {}
        for {set i 1} {$i <= 3} {incr i} {
            if {[string length $hist($setNo,form,entry$i)] > 0} {
                html " <b>" [join $op " "] "</b> "
                html [join $hist($setNo,form,menu$i)] "=" 
                html $hist($setNo,form,entry$i)
                set op $hist($setNo,form,logic$i)
            }
        }
        if {$html3} {
            html {</a><tr>} "\n"
        }
    }
    if {$html3} {
        html {</table><p>}
    } else {
        html {</dl>}
    }
    html "\n"
}

proc displayError {msga msgb} {
    html "<p><center>\n"
    html {<img src="/egwgif/noway.gif" alt="Error">}
    html "<h2>" $msga "</h2>\n"
    if {$msgb != ""} {
        html "<h3>" $msgb "</h3>\n"
    }
    html "</center><p>\n"
}

proc button-europagate {} {
    global useIcons
    html {<a href="http://europagate.dtv.dk/">}
    if {$useIcons} {
        html {<img src="/egwgif/button-egw.gif" alt="Europagate" border=0></a>}
    } else {
        html {Europagate</a> | }
    }
}

proc button-define-target {more} {
    global useIcons
    global env
    global sessionId

    html {<a href="http:} $env(SCRIPT_NAME)
    html / $sessionId {/tform.egw}
    if {$useIcons} {
        html {"><img src="/egwgif/button-define-target.gif" }
	html {alt="Define Target" border=0></a>}
    } else {
        html {">Define Target</a>}
        if {$more} {
            html " | \n"
        } else {
            html "\n"
        }
    }
}

proc button-new-target {more} {
    global useIcons
    global env
    global sessionId
    global mMode

    html {<a href="http:} $env(SCRIPT_NAME)
    html / $sessionId 
    if {$mMode} {
        html {/mtargets.egw}
    } else {
        html {/targets.egw}
    }
    if {$useIcons} {
        html {"><img src="/egwgif/button-new-target.gif" }
	html {alt="New Target" border=0></a>}
    } else {
        html {">New Target</a>}
        if {$more} {
            html " | \n"
        } else {
            html "\n"
        }
    }
}

proc button-view-history {more} {
    global useIcons
    global env
    global sessionId
    global nextSetNo

    html {<a href="http:} $env(SCRIPT_NAME)
    html / $sessionId {/history.egw;}
    catch { html "/" $nextSetNo}
    if {$useIcons} {
        html {"><img src="/egwgif/button-view-history.gif" alt="View History" }
        html {border=0></a>}
    } else {
        html {">View History</a>}
        if {$more} {
            html " | \n"
        } else {
            html "\n"
        }
    }
}

proc button-new-query {more setNo} {
    global useIcons
    global env
    global sessionId
    global hist
    global mMode

    html {<a href="http:} $env(SCRIPT_NAME)
    html / $sessionId 
    if {$mMode} {
        html {/mquery.egw/} $setNo
    } else {
        html {/query.egw/} $hist($setNo,host) + $setNo
    }
    html {">}
    if {$useIcons} {
        html {<img src="/egwgif/button-new-query.gif" }
	html {alt="New Query" border=0></a>}
    } else {
        html {New Query</a>}
        if {$more} {
            html " | \n"
        } else {
            html "\n"
        }
    }
}

proc button-scan-window {more setNo} {
    global useIcons
    global env
    global sessionId
    global hist

    html {<a href="http:} $env(SCRIPT_NAME)
    html / $sessionId {/search.egw/} $setNo + {scan} {">}
    if {$useIcons} {
        html {<img src="/egwgif/button-scan-window.gif" }
	html {alt="Scan" border=0></a>}
    } else {
        html {Scan</a>}
        if {$more} {
            html " | \n"
        } else {
            html "\n"
        }
    }
}

proc maintenance {} {
    html {<hr>This page is maintained by }
    html {<a href="mailto:pwh@dtv.dk"> Peter Wad Hansen </a>.}
    html {Last modified 29. january 1996. <br>}
    html {<em> This and the following pages are under construction and }
    html {will continue to be so until the end of January 1996.</em>}
}

proc splitHostSpec {host} {
    set i [string last . $host]
    if {$i > 1} {
        incr i -1
        return [string range $host 0 $i]
    }
    return $host
}

proc mergeHostSpec {host databases} {
    return ${host}.[join $databases -]
}
